﻿using System;
using Leadtools;
using Leadtools.Annotations.Core;
using Leadtools.Annotations.Designers;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom Draw Designer for the Circle object
   /// It's based off the AnnRectangle Draw Designer
   /// </summary>
   class AnnCircleDrawDesigner : AnnRectangleDrawDesigner
   {
      //We need 2 points, a beginning and an end point
      private LeadPointD _begin = LeadPointD.Empty;
      private LeadPointD _end = LeadPointD.Empty;

      /// <summary>
      /// Contstructor for the AnnCircleDrawDesigner
      /// </summary>
      public AnnCircleDrawDesigner(IAnnAutomationControl automationControl, AnnContainer container, AnnCircleObject annObject)
         : base(automationControl, container, annObject) { }

      /// <summary>
      /// override the pointer down event
      /// set the begin and end to the location of the first click
      /// </summary>
      public override bool OnPointerDown(AnnContainer sender, AnnPointerEventArgs e)
      {
         _begin = e.Location;
         _end = _begin;

         return base.OnPointerDown(sender, e);
      }

      /// <summary>
      /// override the pointer move event
      /// set the new mouse point to the end variable
      /// do some math to create a circle object (width/height stay equal)
      /// </summary>
      public override bool OnPointerMove(AnnContainer sender, AnnPointerEventArgs e)
      {
         _end = e.Location;
         AnnCircleObject circle = (AnnCircleObject)TargetObject;
         double x = (_end.X - _begin.X);
         double y = Math.Abs(_end.Y - _begin.Y);
         double scaleX = 1;
         double scaleY = 1;

         if (x < y)
            scaleX = y / x;
         else
            scaleY = x / y;

         circle.Rect = LeadRectD.Create(_begin.X, _begin.Y, Math.Abs(_end.X - _begin.X) * Math.Abs(scaleX), Math.Abs(_end.Y - _begin.Y) * Math.Abs(scaleY));

         Invalidate(LeadRectD.Empty);
         return true;
      }
   }
}
