﻿using System.Collections.Generic;
using Leadtools.Annotations.Rendering;
using Leadtools.Annotations.Automation;
using Leadtools.Annotations.Core;
using Leadtools;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom renderer Designer for the Circle object
   /// It's based off the Ann Ellipse renderer
   /// </summary>
   public class AnnCircleObjectRenderer : AnnEllipseObjectRenderer
   {
      /// <summary>
      /// Constructor for the renderer
      /// get the ellipse object renderer and use the same styles for the circle
      /// </summary>
      public AnnCircleObjectRenderer(AnnAutomationManager manager)
         : base()
      {
         IAnnObjectRenderer annEllipseObjRenderer = manager.RenderingEngine.Renderers[AnnObject.EllipseObjectId];
         LabelRenderer = annEllipseObjRenderer.LabelRenderer;
         LocationsThumbStyle = annEllipseObjRenderer.LocationsThumbStyle;
         RotateCenterThumbStyle = annEllipseObjRenderer.RotateCenterThumbStyle;
         RotateGripperThumbStyle = annEllipseObjRenderer.RotateGripperThumbStyle;
      }

      /// <summary>
      /// override the RenderThumbs method
      /// go through the thumbs and remove the top bottom left and right thumbs so it can't be changed from a circle
      /// </summary>
      public override void RenderThumbs(AnnContainerMapper mapper, LeadPointD[] thumbLocations, AnnFixedStateOperations operations)
      {
         List<LeadPointD> newThumbs = new List<LeadPointD>();
         for (int i = 0; i < thumbLocations.Length; i += 2)
            newThumbs.Add(thumbLocations[i]);

         base.RenderThumbs(mapper, newThumbs.ToArray(), operations);
      }
   }
}
