﻿using System;
using Leadtools;
using Leadtools.Annotations.Core;
using Leadtools.Annotations.Designers;
using System.Windows.Forms;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom Draw Designer for the number object
   /// It's based off the generic Draw Designer
   /// </summary>
   class AnnNumberDrawDesigner : AnnRectangleDrawDesigner
   {
      /// <summary>
      /// Contstructor for the AnnNumberDrawDesigner
      /// </summary>
      public AnnNumberDrawDesigner(IAnnAutomationControl automationControl, AnnContainer container, AnnNumberObject annObject)
         : base(automationControl, container, annObject)
      { }

      /// <summary>
      /// override the pointer down event
      /// see how many other Number objects are in the container and iterate by 1 for each one found
      /// set the text to the number
      /// </summary>
      public override bool OnPointerDown(AnnContainer sender, AnnPointerEventArgs e)
      {
         AnnNumberObject numberObj = (AnnNumberObject)TargetObject;
         var handled = base.OnPointerDown(sender, e);
         int num = 0;

         foreach (AnnObject obj in Container.Children)
         {
            if (obj.Id == AnnNumberObject.NumberObjectId)
            {
               num++;
            }
         }
         numberObj.Text = num.ToString();

         handled = true;

         return handled;
      }

      /// <summary>
      /// override the pointer move event
      /// we want the font size to increase when drawing the number object
      /// make sure the font size is never less than 1
      /// set the font size to a fraction of the width 
      /// set the number objects height to make sure the number is visible at all times
      /// </summary>
      public override bool OnPointerMove(AnnContainer sender, AnnPointerEventArgs e)
      {
         var handled = base.OnPointerMove(sender, e);
         AnnNumberObject numberObj = (AnnNumberObject)TargetObject;
         if (numberObj.Font.FontSize != 0)
         {
            var newFont = Container.Mapper.RectFromContainerCoordinates(numberObj.Bounds, AnnFixedStateOperations.None).Width * .5;
            if (newFont < 1)
               numberObj.Font.FontSize = 1;
            else
               numberObj.Font.FontSize = newFont;

            var textSize = TextRenderer.MeasureText(numberObj.Text, new System.Drawing.Font(numberObj.Font.FontFamilyName, (float)numberObj.Font.FontSize));
            var contTextSize = Container.Mapper.SizeToContainerCoordinates(LeadSizeD.Create(textSize.Width, textSize.Height));
            if(contTextSize.Height != 0)
               numberObj.Rect = LeadRectD.Create(numberObj.Rect.X, numberObj.Rect.Y, numberObj.Rect.Width, contTextSize.Height);

            Invalidate(LeadRectD.Empty);
         }
         return handled;
      }
   }
}
