﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Leadtools.Annotations.Designers;
using Leadtools.Annotations.Core;
using Leadtools;
using System.Windows.Forms;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom Edit Designer for the number object
   /// It's based off the text edit Designer
   /// </summary>
   class AnnNumberEditDesigner : AnnTextEditDesigner
   {
      LeadRectD beforeRect = LeadRectD.Empty;
      double fontSize;

      /// <summary>
      /// Constructor for the Number Edit Designer
      /// </summary>
      public AnnNumberEditDesigner(IAnnAutomationControl automationControl, AnnContainer container, AnnNumberObject annObject)
       : base(automationControl, container, annObject)
      { }

      /// <summary>
      /// Override the OnEdit event
      /// verify the before rectangle isn't empty
      /// check to see if the width is changing
      /// do some math to calculate the new font size
      /// do some math to calculate the new rectangle size
      /// </summary>
      protected override void OnEdit(AnnEditDesignerEventArgs e)
      {
         base.OnEdit(e);

         if (beforeRect != LeadRectD.Empty)
         {
            AnnNumberObject numberObj = (AnnNumberObject)TargetObject;

            if (TargetObject.Bounds.Width != beforeRect.Width)
            {
               double widthRatio = TargetObject.Bounds.Width / beforeRect.Width;
               if (widthRatio != 0)
               {
                  numberObj.Font.FontSize = fontSize * widthRatio;
               }
            }

            var textSize = TextRenderer.MeasureText(numberObj.Text, new System.Drawing.Font(numberObj.Font.FontFamilyName, (float)numberObj.Font.FontSize));
            var contTextSize = Container.Mapper.SizeToContainerCoordinates(LeadSizeD.Create(textSize.Width, textSize.Height));
            if (numberObj.Bounds.Height < contTextSize.Height)
               numberObj.Rect = LeadRectD.Create(numberObj.Rect.X, numberObj.Rect.Y, numberObj.Rect.Width, contTextSize.Height);
         }
      }

      /// <summary>
      /// Override the on pointer down event
      /// get the before edit rectangle and font size
      /// </summary>
      public override bool OnPointerDown(AnnContainer sender, AnnPointerEventArgs e)
      {
         var handled = base.OnPointerDown(sender, e);
         beforeRect = TargetObject.Bounds;
         fontSize = ((AnnNumberObject)TargetObject).Font.FontSize;

         return handled;
      }
   }
}
