﻿using System;
using Leadtools.Annotations.Core;
using Leadtools.Annotations.Designers;
using Leadtools;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom Draw Designer for the star object
   /// It's based off the Ann Polyline Draw Designer
   /// </summary>
   class AnnStarDrawDesigner : AnnPolylineDrawDesigner
   {
      /// <summary>
      /// Contstructor for the AnnCircleDrawDesigner
      /// </summary>
      public AnnStarDrawDesigner(IAnnAutomationControl automationControl, AnnContainer container, AnnStarObject annObject)
         : base(automationControl, container, annObject)
      { }
      LeadPointD center = LeadPointD.Empty;
      /// <summary>
      /// override the pointer down event
      /// get the location of the click for the center of the star
      /// </summary>
      public override bool OnPointerDown(AnnContainer sender, AnnPointerEventArgs e)
      {
         var handled = base.OnPointerDown(sender, e);

         center = e.Location;

         return handled;
      }

      /// <summary>
      /// override the pointer up event
      /// tell the designer to stop working and invalidate
      /// </summary>
      public override bool OnPointerUp(AnnContainer sender, AnnPointerEventArgs e)
      {
         bool handled = base.OnPointerUp(sender, e);
         EndWorking();
         Invalidate(LeadRectD.Empty);
         handled = true;
         return handled;
      }

      /// <summary>
      /// override the pointer move event
      /// we are drawing the star while moving the mouse
      /// use the distance formula to get the radius of the star (in container coordinates)
      /// use more math to determine the locations of the 10 points of the star
      /// </summary>
      public override bool OnPointerMove(AnnContainer sender, AnnPointerEventArgs e)
      {
         var handled = base.OnPointerMove(sender, e);

         if (center != LeadPointD.Empty)
         {
            TargetObject.Points.Clear();
            var alpha = (2 * Math.PI) / 10;
            var radius = Math.Sqrt(Math.Pow((e.Location.X - center.X), 2) + Math.Pow((e.Location.Y - center.Y), 2));

            for (int i = 10; i != 0; i--)
            {
               var r = radius * (i % 2 + 1) / 2;
               var omega = alpha * i;
               LeadPointD point = LeadPointD.Create(center.X + (r * Math.Sin(omega)), center.Y + (r * Math.Cos(omega)));
               TargetObject.Points.Add(point);
            }
         }

         return handled;
      }
   }
}
