﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Leadtools.Annotations.Core;
using Leadtools.Annotations.Designers;
using Leadtools;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// This class is the custom Draw Designer for the tiny stamp object
   /// It's based off the generic Draw Designer
   /// </summary>
   class AnnTinyStampDrawDesigner : AnnDrawDesigner
   {
      /// <summary>
      /// Contstructor for the AnnCircleDrawDesigner
      /// </summary>
      public AnnTinyStampDrawDesigner(IAnnAutomationControl automationControl, AnnContainer container, AnnTinyStampObject annObject)
         : base(automationControl, container, annObject)
      { }

      /// <summary>
      /// override the pointer down event
      /// we want the image to only be 50x50 converted to container coordinates
      /// The location of the click will be the center
      /// do some math to create the points and add them to the object
      /// </summary>
      public override bool OnPointerDown(AnnContainer sender, AnnPointerEventArgs e)
      {
         var handled = base.OnPointerDown(sender, e);

         LeadSizeD imageSize = Container.Mapper.SizeToContainerCoordinates(LeadSizeD.Create(50, 50));

         if (StartWorking())
         {
            LeadPointD point1 = LeadPointD.Create(e.Location.X - imageSize.Width, e.Location.Y + imageSize.Height);
            LeadPointD point2 = LeadPointD.Create(e.Location.X + imageSize.Width, e.Location.Y + imageSize.Height);
            LeadPointD point3 = LeadPointD.Create(e.Location.X + imageSize.Width, e.Location.Y - imageSize.Height);
            LeadPointD point4 = LeadPointD.Create(e.Location.X - imageSize.Width, e.Location.Y - imageSize.Height);

            TargetObject.Points.Add(point1);
            TargetObject.Points.Add(point2);
            TargetObject.Points.Add(point3);
            TargetObject.Points.Add(point4);
         }
         handled = true;

         return handled;
      }


      /// <summary>
      /// override the pointer up event
      /// tell the designer to stop working and invalidate
      /// </summary>
      public override bool OnPointerUp(AnnContainer sender, AnnPointerEventArgs e)
      {
         bool handled = base.OnPointerUp(sender, e);

         handled = true;
         EndWorking();
         Invalidate(TargetObject.Bounds);

         return true;
      }
   }
}
