using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using Leadtools;
using Leadtools.Codecs;
using Leadtools.Controls;
using Leadtools.Demos;
using Leadtools.Demos.Dialogs;
using Leadtools.Annotations.Core;
using Leadtools.Annotations.Automation;
using Leadtools.Annotations.WinForms;
using Leadtools.Annotations.Designers;
using Leadtools.Annotations.Rendering;

namespace CustomAnnotationsDemo
{
   /// <summary>
   /// Summary description for MainForm.
   /// </summary>
   public class MainForm : Form
   {
      private MainMenu _mmMain;
      private MenuItem _miFile;
      private MenuItem _miFileOpen;
      private MenuItem _miFileExit;
      private MenuItem _miFileSep1;
      private MenuItem _miHelp;
      private MenuItem _miHelpAbout;
      private MenuItem _miHelpDisplay;
      private IContainer components;

      public MainForm()
      {
         //
         // Required for Windows Form Designer support
         //
         InitializeComponent();

         //
         // TODO: Add any constructor code after InitializeComponent call
         //
      }

      /// <summary>
      /// Clean up any resources being used.
      /// </summary>
      protected override void Dispose(bool disposing)
      {
         if (disposing)
         {
            if (components != null)
            {
               components.Dispose();
            }
         }

         if (_viewer != null)
         {
            if (_viewer.Image != null)
               _viewer.Image.Dispose();
            _viewer.Dispose();
         }

         if (_codecs != null)
            _codecs.Dispose();

         base.Dispose(disposing);
      }

      #region Windows Form Designer generated code
      /// <summary>
      /// Required method for Designer support - do not modify
      /// the contents of this method with the code editor.
      /// </summary>
      private void InitializeComponent()
      {
         components = new Container();
         ComponentResourceManager resources = new ComponentResourceManager(typeof(MainForm));
         _mmMain = new MainMenu(components);
         _miFile = new MenuItem();
         _miFileOpen = new MenuItem();
         _miFileSep1 = new MenuItem();
         _miFileExit = new MenuItem();
         _miHelp = new MenuItem();
         _miHelpAbout = new MenuItem();
         _miHelpDisplay = new MenuItem();
         SuspendLayout();
         // 
         // _mmMain
         // 
         _mmMain.MenuItems.AddRange(new MenuItem[] {
            _miFile,
            _miHelp});
         // 
         // _miFile
         // 
         _miFile.Index = 0;
         _miFile.MenuItems.AddRange(new MenuItem[] {
            _miFileOpen,
            _miFileSep1,
            _miFileExit});
         _miFile.Text = "&File";
         // 
         // _miFileOpen
         // 
         _miFileOpen.Index = 0;
         _miFileOpen.Shortcut = Shortcut.CtrlO;
         _miFileOpen.Text = "&Open...";
         _miFileOpen.Click += new EventHandler(_miFileOpen_Click);
         // 
         // _miFileSep1
         // 
         _miFileSep1.Index = 1;
         _miFileSep1.Text = "-";
         // 
         // _miFileExit
         // 
         _miFileExit.Index = 2;
         _miFileExit.Text = "E&xit";
         _miFileExit.Click += new EventHandler(_miFileExit_Click);
         // 
         // _miHelp
         // 
         _miHelp.Index = 1;
         _miHelp.MenuItems.AddRange(new MenuItem[] {
            _miHelpAbout,
            _miHelpDisplay});
         _miHelp.Text = "&Help";
         // 
         // _miHelpAbout
         // 
         _miHelpAbout.Index = 0;
         _miHelpAbout.Text = "&About...";
         _miHelpAbout.Click += new EventHandler(_miHelpAbout_Click);
         // 
         // _miHelpDisplay
         // 
         _miHelpDisplay.Index = 0;
         _miHelpDisplay.Text = "&Display Help...";
         _miHelpDisplay.Click += new EventHandler(_miHelpDisplay_Click);
         // 
         // MainForm
         // 
         AutoScaleBaseSize = new Size(5, 13);
         ClientSize = new Size(800, 500);
         Icon = ((Icon)(resources.GetObject("$Icon")));
         Menu = _mmMain;
         Name = "MainForm";
         StartPosition = FormStartPosition.CenterScreen;
         Text = "Custom Annotation Demo";
         Load += new EventHandler(MainForm_Load);
         ResumeLayout(false);

      }
      #endregion

      /// <summary>
      /// The main entry point for the application.
      /// </summary>
      [STAThread]
      static void Main()
      {
         Support.SetLicense();

         if (RasterSupport.IsLocked(RasterSupportType.Document))
         {
            MessageBox.Show(String.Format("{0} Support is locked!", RasterSupportType.Document.ToString()), "Warning");
            return;
         }

         try
         {
            Application.Run(new MainForm());
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message);
         }
      }

      private ImageViewer _viewer;
      private RasterCodecs _codecs;
      private AutomationInteractiveMode _annInteractiveMode;
      private IAnnAutomationControl _automationControl;
      private AnnAutomationManager _annManager;
      private AnnAutomation _annAutomation;

      /// <summary>
      /// Initialize the Application.
      /// </summary>
      private void MainForm_Load(object sender, EventArgs e)
      {
         Messager.Caption = "C# Custom Annotation Demo";
         Text = Messager.Caption;

         // initialize the _viewer object
         _viewer = new ImageViewer();
         _viewer.Dock = DockStyle.Fill;
         _viewer.BackColor = Color.DarkGray;
         Controls.Add(_viewer);
         _viewer.BringToFront();

         InitAnnotations();

         //create a plain white background to draw on
         _viewer.Image = RasterImage.Create(1000, 1000, 24, 96, RasterColor.White);

         // initialize the codecs object.
         _codecs = new RasterCodecs();

         DisplayHelpMessage();
      }

      private void DisplayHelpMessage()
      {
         string displayMessage = "This demo shows how to create different Custom Annotation objects.\n" +
   "In this demo there are 4 different objects:\n\n" +
   "AnnCircleObject\nThis object is based on the Ellipse object and allows the user to draw a perfect circle\n\n" +
   "AnnNumberObject\nThis object is based on the text object. It allows the user to add a number to the image which increments each time a new one is added. When editing this object, the font size will change as well\n\n" +
   "AnnStarObject\nThis object is based on the Polyline Object. It allows the user to click and drag the mouse to draw a perfect Star\n\n" +
   "AnnTinyStampObject\nThis object is based on the Stamp Object. It allows the user to click once and add a non-resizeable stamp with an image\n\n";

         MessageBox.Show(this, displayMessage, Text);
      }

      /// <summary>
      /// Load a new image
      /// </summary>
      private void _miFileOpen_Click(object sender, EventArgs e)
      {
         try
         {
            // load the image
            ImageFileLoader loader = new ImageFileLoader();

            loader.ShowLoadPagesDialog = true;
            if (loader.Load(this, _codecs, true) > 0)
            {
               // update the caption
               Text = string.Format("{0} - {1}", loader.FileName, Messager.Caption);

               if (_annAutomation != null && _annAutomation.Container.Children.Count > 0)
                  _annAutomation.Container.Children.Clear();
               // set the new image in the viewer.
               _viewer.Image = loader.Image;
            }
         }
         catch (Exception ex)
         {
            Messager.ShowError(this, ex);
         }
      }

      /// <summary>
      /// Shutdown
      /// </summary>
      private void _miFileExit_Click(object sender, EventArgs e)
      {
         Close();
      }

      /// <summary>
      /// show the about dialog
      /// </summary>
      private void _miHelpAbout_Click(object sender, EventArgs e)
      {
         using (AboutDialog aboutDialog = new AboutDialog("Custom Annotation ", ProgrammingInterface.CS))
            aboutDialog.ShowDialog(this);
      }

      /// <summary>
      /// show the help message dialog
      /// </summary>
      private void _miHelpDisplay_Click(object sender, EventArgs e)
      {
         DisplayHelpMessage();
      }

      /// <summary>
      /// initialize the annotation objects
      /// </summary>
      void InitAnnotations()
      {
         //AnnManager initialization comes first
         _annManager = new AnnAutomationManager();
         _annManager.RestrictDesigners = true;
         _annManager.RenderingEngine = new AnnWinFormsRenderingEngine();

         //Create my custom objects 
         CreateCustomObject(_annManager);

         //create the helper and toolbar then add it to the form
         AutomationManagerHelper annHelper = new AutomationManagerHelper(_annManager);
         annHelper.CreateToolBar();
         Controls.Add(annHelper.ToolBar);

         //create the automation control and attach the viewer to it
         _automationControl = new ImageViewerAutomationControl();
         ((ImageViewerAutomationControl)_automationControl).ImageViewer = _viewer;

         //initialize the interactive mode and add it to the viewer
         _annInteractiveMode = new AutomationInteractiveMode();
         _annInteractiveMode.AutomationControl = _automationControl;
         _viewer.InteractiveModes.BeginUpdate();
         _viewer.InteractiveModes.Add(_annInteractiveMode);
         _viewer.InteractiveModes.EndUpdate();

         //initialize the automation and events
         _annAutomation = new AnnAutomation(_annManager, _automationControl);
         _annAutomation.OnShowContextMenu += automation_OnShowContextMenu;
         _annAutomation.OnShowObjectProperties += _annAutomation_OnShowObjectProperties;
         _annAutomation.UnlockObject += _annAutomation_UnlockObject;
         _annAutomation.LockObject += _annAutomation_LockObject;

         //whenever we load a new image, set the new size in the container
         _viewer.ItemChanged += (s, e) =>
         {
            if (e.Reason == ImageViewerItemChangedReason.Image)
            {
               _annAutomation.Container.Size = _annAutomation.Container.Mapper.SizeToContainerCoordinates(_viewer.ImageSize.ToLeadSizeD());
            }
         };
         _annAutomation.Active = true;
      }

      /// <summary>
      /// create all the custom objects
      /// </summary>
      /// <param name="annManager"></param>
      private void CreateCustomObject(AnnAutomationManager annManager)
      {
         //create the circle object, assign the designers, set the toolbar image, and then add it to the manager and rendering engine
         AnnAutomationObject circleAutomationObject = new AnnAutomationObject();
         circleAutomationObject.Id = AnnCircleObject.CircleObjectId;
         circleAutomationObject.Name = "Circle";
         circleAutomationObject.ToolBarToolTipText = circleAutomationObject.Name;
         circleAutomationObject.DrawDesignerType = typeof(AnnCircleDrawDesigner);
         circleAutomationObject.EditDesignerType = typeof(AnnRectangleEditDesigner);
         circleAutomationObject.RunDesignerType = typeof(AnnRunDesigner);
         circleAutomationObject.ObjectTemplate = new AnnCircleObject();
         circleAutomationObject.ToolBarImage = new Bitmap(typeof(MainForm), "Resources.CircleIcon.png");
         annManager.Objects.Add(circleAutomationObject);
         annManager.RenderingEngine.Renderers.Add(AnnCircleObject.CircleObjectId, new AnnCircleObjectRenderer(_annManager));

         //create the tiny stamp object, assign the designers, set the toolbar image, and then add it to the manager and rendering engine
         AnnAutomationObject tinyStampAutomationObject = new AnnAutomationObject();
         tinyStampAutomationObject.Id = AnnTinyStampObject.TinyStampId;
         tinyStampAutomationObject.Name = "TinyStamp";
         tinyStampAutomationObject.ToolBarToolTipText = tinyStampAutomationObject.Name;
         tinyStampAutomationObject.DrawDesignerType = typeof(AnnTinyStampDrawDesigner);
         tinyStampAutomationObject.EditDesignerType = typeof(AnnRectangleEditDesigner);
         tinyStampAutomationObject.RunDesignerType = typeof(AnnRunDesigner);
         AnnTinyStampObject tinyStamp = new AnnTinyStampObject();
         Image img = new Bitmap(System.Reflection.Assembly.GetEntryAssembly().GetManifestResourceStream("CustomAnnotationsDemo.Resources.Smile.png"));
         tinyStamp.Picture = new AnnPicture((byte[])new ImageConverter().ConvertTo(img, typeof(byte[])));
         tinyStampAutomationObject.ObjectTemplate = tinyStamp;
         tinyStampAutomationObject.ToolBarImage = new Bitmap(typeof(MainForm), "Resources.TinyStampIcon.png");
         annManager.Objects.Add(tinyStampAutomationObject);
         annManager.RenderingEngine.Renderers.Add(AnnTinyStampObject.TinyStampId, new AnnTinyStampRenderer(_annManager));

         //create the star object, assign the designers, set the toolbar image, and then add it to the manager and rendering engine
         AnnAutomationObject starAutomationObject = new AnnAutomationObject();
         starAutomationObject.Id = AnnStarObject.StarObjectId;
         starAutomationObject.Name = "Star";
         starAutomationObject.ToolBarToolTipText = starAutomationObject.Name;
         starAutomationObject.DrawDesignerType = typeof(AnnStarDrawDesigner);
         starAutomationObject.EditDesignerType = typeof(AnnPolylineEditDesigner);
         starAutomationObject.RunDesignerType = typeof(AnnRunDesigner);
         starAutomationObject.ObjectTemplate = new AnnStarObject();
         starAutomationObject.ToolBarImage = new Bitmap(typeof(MainForm), "Resources.StarIcon.png");
         annManager.Objects.Add(starAutomationObject);
         annManager.RenderingEngine.Renderers.Add(AnnStarObject.StarObjectId, new AnnStarObjectRenderer(_annManager));

         //create the number object, assign the designers, set the toolbar image, and then add it to the manager and rendering engine
         AnnAutomationObject numberAutomationObject = new AnnAutomationObject();
         numberAutomationObject.Id = AnnNumberObject.NumberObjectId;
         numberAutomationObject.Name = "Number";
         numberAutomationObject.ToolBarToolTipText = numberAutomationObject.Name;
         numberAutomationObject.DrawDesignerType = typeof(AnnNumberDrawDesigner);
         numberAutomationObject.EditDesignerType = typeof(AnnNumberEditDesigner);
         numberAutomationObject.RunDesignerType = typeof(AnnRunDesigner);
         numberAutomationObject.ObjectTemplate = new AnnNumberObject();
         numberAutomationObject.ToolBarImage = new Bitmap(typeof(MainForm), "Resources.NumberIcon.png");
         annManager.Objects.Add(numberAutomationObject);
         annManager.RenderingEngine.Renderers.Add(AnnNumberObject.NumberObjectId, new AnnNumberObjectRenderer(_annManager));
      }

      /// <summary>
      /// event when right clicking 
      /// </summary>
      void automation_OnShowContextMenu(object sender, AnnAutomationEventArgs e)
      {
         if (e != null && e.Object != null)
         {
            _automationControl.AutomationInvalidate(LeadRectD.Empty);
            AnnAutomationObject annAutomationObject = e.Object as AnnAutomationObject;
            if (annAutomationObject != null)
            {
               ObjectContextMenu menu = annAutomationObject.ContextMenu as ObjectContextMenu;
               if (menu != null)
               {
                  menu.Automation = sender as AnnAutomation;
                  menu.Show(this, _viewer.PointToClient(Cursor.Position));
               }
            }
         }
         else
         {
            ManagerContextMenu defaultMenu = new ManagerContextMenu();
            defaultMenu.Automation = sender as AnnAutomation;
            defaultMenu.Show(this, _viewer.PointToClient(Cursor.Position));
         }
      }

      /// <summary>
      /// event when clicking on poperties in the context menu
      /// </summary>
      void _annAutomation_OnShowObjectProperties(object sender, AnnAutomationEventArgs e)
      {
         using (var dlg = new AutomationUpdateObjectDialog())
         {
            var currentObject = _annAutomation.CurrentEditObject;
            var isSelectionObject = currentObject is AnnSelectionObject;

            if (currentObject != null)
            {
               if (!currentObject.SupportsContent || isSelectionObject)
               {
                  dlg.SetPageVisible(AutomationUpdateObjectDialogPage.Content, false);
                  if (isSelectionObject)
                  {
                     dlg.SetPageVisible(AutomationUpdateObjectDialogPage.Reviews, false);
                  }
               }
            }

            dlg.Automation = _annAutomation;

            try
            {
               dlg.ShowDialog(this);
               e.Cancel = !dlg.IsModified;
            }
            catch (Exception ex)
            {
               MessageBox.Show(ex.Message);
            }
         }
      }

      /// <summary>
      /// event when clicking on 'Unlock' in context menu
      /// </summary>
      void _annAutomation_UnlockObject(object sender, AnnLockObjectEventArgs e)
      {
         e.Cancel = true;
         //PasswordDialog
         AutomationPasswordDialog passwordDialog = new AutomationPasswordDialog();
         if (passwordDialog.ShowDialog(this) == DialogResult.OK)
         {
            e.Object.Unlock(passwordDialog.Password);
            if (e.Object.IsLocked)
            {
               MessageBox.Show("You've entered an invalid password.", "Invalid Password", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            }

            _annAutomation.Invalidate(LeadRectD.Empty);
         }
      }

      /// <summary>
      /// event when clicking on 'lock' in the context menu
      /// </summary>
      void _annAutomation_LockObject(object sender, AnnLockObjectEventArgs e)
      {
         AutomationPasswordDialog passwordDialog = new AutomationPasswordDialog();
         passwordDialog.Lock = true;
         if (passwordDialog.ShowDialog(this) == DialogResult.OK)
         {
            e.Password = passwordDialog.Password;
         }
         else
            e.Cancel = true;
      }
   }
}
